from inspect import cleandoc

_DESC = """
    The following commands provide local support in the continuous
    integration flow.
"""

meta = {
    'help': 'continuous integration utilities',
    'description': cleandoc(_DESC),
}
